import {Select} from '../../builder/type/Select.ts'

/**
 * Query builder SELECT clause, with added features for working with models.
 * @extends Select
 */
export class ModelSelect<T> extends Select<T> {
    /**
     * List of relations to eager load.
     * @type Array<string>
     */
    protected _withs: string[] = []

    /**
     * Eager load a relation.
     * @example posts
     * @example posts.comments
     * @param {string} related
     * @return self
     */
    public with(related: string) {
        this._withs.push(related)
        return this
    }

    /**
     * Get the relations to eager load.
     * @type Array<string>
     */
    public get eager_relations(): string[] {
        return [...this._withs]
    }

    /**
     * Make a copy of this query.e
     * @return ModelSelect
     */
    clone(): ModelSelect<T> {
        const select = super.clone() as ModelSelect<T>
        select._withs = this._withs
        return select
    }
}
